@@ -348,6 +348,51 @@ context 'Manpage' do
       assert_includes output, %(Oh, here it goes again\nI should have known,\nshould have known,\nshould have known again)
     end
 
+    test 'should preserve break between paragraphs in normal table cell' do
+      input = <<~EOS.chop
+      #{SAMPLE_MANPAGE_HEADER}
+
+      [cols=3*]
+      |===
+      |single paragraph
+      |first paragraph
+
+      second paragraph
+      |foo
+
+      more foo
+
+      even more foo
+      |===
+      EOS
+
+      expected_coda = <<~'EOS'.chop
+      .TS
+      allbox tab(:);
+      lt lt lt.
+      T{
+      .sp
+      single paragraph
+      T}:T{
+      .sp
+      first paragraph
+      .sp
+      second paragraph
+      T}:T{
+      .sp
+      foo
+      .sp
+      more foo
+      .sp
+      even more foo
+      T}
+      .TE
+      .sp
+      EOS
+      output = Asciidoctor.convert input, backend: :manpage
+      assert output.end_with? expected_coda
+    end
+
     test 'should normalize whitespace in a list item' do
       input = <<~EOS.chop
       #{SAMPLE_MANPAGE_HEADER}
@@ -660,11 +705,13 @@ context 'Manpage' do
       EOS
       expected_coda = <<~'EOS'.chop
       allbox tab(:);
-      lt.
+      ltB.
       T{
       .sp
       Header
       T}
+      .T&
+      lt.
       T{
       .sp
       Body 1
@@ -720,7 +767,7 @@ context 'Manpage' do
       .B Table 1. Table of options
       .TS
       allbox tab(:);
-      lt lt lt.
+      ltB ltB ltB.
       T{
       .sp
       Name
@@ -731,6 +778,8 @@ context 'Manpage' do
       .sp
       Default
       T}
+      .T&
+      lt lt lt.
       T{
       .sp
       dim
